#ifndef _OBJ_UTILS_H
#define _OBJ_UTILS_H

char error_list[][32]={"TOO MANY VERTEXES", "MISSING FACE LIST", "BAD FACE INDEX", "NO END", "DUPLICATE", "NOT ENOUGH TESSILATIONS"};

void tessilate_sphere(Vect3D &, int, int, int, int, Poly_List &);
void tessilate_cone(Vect3D &, Vect3D &, int, int, int, int, Poly_List &);

//********************************************************
inline void error(int obj_num, int err) {
	cout << "\n\t\t MALFORMED OBJECT  < " << obj_num << " >\n";
	cout << "\t\t  - " <<  error_list[err] << endl;
}
inline void skip_white(ifstream &in, char *buf) {
	do { 
		in.getline(buf, 128, '\n');
	} while(buf[0]==0  ||  buf[0]=='#');
}
int count_commas(char *buf) {
	int count=0;
	for(unsigned int i=0; i<strlen(buf); i++)
		if(buf[i]==',') count++;
	return count;
}
int next_comma(char *buf, int pos) {
	int comma_pos=0;
	for(unsigned int i=pos; i<strlen(buf); i++)
		if(buf[i]==',') {	comma_pos=i;	break; }
	return comma_pos;
}
bool duplicate(Poly &poly, Poly_List &polygons) {
	for(poly_Iter=polygons.begin(); poly_Iter!=polygons.end(); ++poly_Iter)
		if(poly==(*poly_Iter)) return true;
	return false;
}
//********************************************************

void tessilate_sphere(Vect3D &center, int radius, int degrees, int vert, int horiz, Poly_List &polygons) {
	Poly curr_poly;
	Vect3D n_pole;
	Matrix rotator;
	double cur_y, cur_y2, cur_func, cur_sqrt, step_v, arc_length;
	int radius_2, curr_rotate, rotate_step;
	int offsetA, offsetB, offsetC, offsetD, index, index2;

	radius_2=radius*radius;
	curr_rotate=rotate_step=360/horiz;
	n_pole.set(center.getx(), center.gety()+radius, center.getz());
	arc_length=float(degrees)/360.0F;
	step_v = (float(radius)*arc_length) / ((float(vert)-1)/2.0F);
	Vect3D tessilation[horiz][vert];

	cur_y=0;
	for(index=0; index<vert; index++) {
		cur_y=radius-(index*step_v);
		cur_func=radius_2-(cur_y*cur_y);
		if(cur_func>-0.1 && cur_func<0.1)
			cur_func=0;
		if(cur_func<0) {
			cur_y2=radius-(cur_y-radius);
			cur_func=radius_2-(cur_y2*cur_y2);
		}
		cur_sqrt=sqrt(cur_func);
		tessilation[0][index].setx(n_pole.getx() + cur_sqrt);
		tessilation[0][index].sety(n_pole.gety() - cur_y);
		tessilation[0][index].setz(n_pole.getz());
	}

	for(index=1; index<horiz; index++) {
		rotator.setIdentity();
		rotator.rotate(Vect3D(0,curr_rotate, 0));
		curr_rotate+=rotate_step;
		for(index2=0; index2<vert; index2++) {
			tessilation[index][index2]=rotator.cross(tessilation[0][index2]-center);
			tessilation[index][index2]+=center;
		}	
	}


	for(index=horiz-1; index>-1; index--) {
		offsetA=index;
		offsetB=(index+1) % horiz;
		for(index2=0; index2<vert; index2++) {
			offsetC=index2;
			offsetD=((index2+1)==vert) ? 0 : index2+1;
			curr_poly.add(tessilation[offsetA][offsetD]);
			curr_poly.add(tessilation[offsetB][offsetD]);
			curr_poly.add(tessilation[offsetB][offsetC]);
			curr_poly.add(tessilation[offsetA][offsetC]);
			curr_poly.solve();
			polygons.push_back(curr_poly);
			curr_poly.clear();
		}
	}
}


void tessilate_cone(Vect3D &center, Vect3D &orientation, int radius, int radius2, int height, int vert, Poly_List &polygons) {
	Poly curr_poly;
	Vect3D n_pole;
	Matrix rotator;
	double cur_y, cur_y2, cur_func, cur_sqrt, step_v;
	int radius_2, half_limit, index, index2;
	int offsetA, offsetB;

	rotator.rotate(orientation);
	radius_2=radius*radius;
	n_pole.set(center.getx(), center.gety()+radius, center.getz());
  half_limit=(vert/2);
	step_v = (radius*2) / half_limit;
	Vect3D tessilation[2][vert];

	cur_y=0;
	for(index=0; index<half_limit+1; index++) {
		cur_y=radius-(index*step_v);
		cur_func=radius_2-(cur_y*cur_y);
		if(cur_func>-0.1 && cur_func<0.1)
			cur_func=0;
		if(cur_func<0) {
			cur_y2=radius-(cur_y-radius);
			cur_func=radius_2-(cur_y2*cur_y2);
		}
		cur_sqrt=sqrt(cur_func);
		tessilation[0][index].setx(n_pole.getx() + cur_sqrt);
		tessilation[0][index].sety(n_pole.gety() - cur_y);
		tessilation[0][index].setz(n_pole.getz());
		if(index>0 && index<half_limit) {
			tessilation[0][vert-index].setx(n_pole.getx() - (tessilation[0][index].getx()-n_pole.getx()) );
			tessilation[0][vert-index].sety(tessilation[0][index].gety());
			tessilation[0][vert-index].setz(n_pole.getz());
		}
	}

	radius_2=radius2*radius2;
	n_pole.set(center.getx(), center.gety()+radius2+(radius-radius2), center.getz()+height);
	step_v = (radius2*2) / half_limit;

	cur_y=0;
	for(index=0; index<half_limit+1; index++) {
		cur_y=radius2-(index*step_v);
		cur_func=radius_2-(cur_y*cur_y);
		if(cur_func>-0.1 && cur_func<0.1)
			cur_func=0;
		if(cur_func<0) {
			cur_y2=radius2-(cur_y-radius2);
			cur_func=radius_2-(cur_y2*cur_y2);
		}
		cur_sqrt=sqrt(cur_func);
		tessilation[1][index].setx(n_pole.getx() + cur_sqrt);
		tessilation[1][index].sety(n_pole.gety() - cur_y);
		tessilation[1][index].setz(n_pole.getz());
		if(index>0 && index<half_limit) {
			tessilation[1][vert-index].setx(n_pole.getx() - (tessilation[1][index].getx()-n_pole.getx()) );
			tessilation[1][vert-index].sety(tessilation[1][index].gety());
			tessilation[1][vert-index].setz(n_pole.getz());
		}
	}

	for(index=0; index<2; index++) {
		for(index2=0; index2<vert; index2++) {
			tessilation[index][index2]=rotator.cross(tessilation[index][index2]-center);
			tessilation[index][index2]+=center;
		}
	}

	for(index=0; index<vert; index++)
		curr_poly.us_add(tessilation[1][index]);
	curr_poly.solve();
	polygons.push_back(curr_poly);
	curr_poly.clear();

	for(index=vert-1; index>-1; index--)
		curr_poly.us_add(tessilation[0][index]);
	curr_poly.solve();
	polygons.push_back(curr_poly);
	curr_poly.clear();

	for(index=0; index<vert; index++) {
		offsetA=index;
		offsetB=(index+1) % vert;
		curr_poly.us_add(tessilation[1][offsetA]);
		curr_poly.us_add(tessilation[0][offsetA]);
		curr_poly.us_add(tessilation[0][offsetB]);
		curr_poly.us_add(tessilation[1][offsetB]);
		curr_poly.solve();
		polygons.push_back(curr_poly);
		curr_poly.clear();
	}
}

#endif