#ifndef _VECT3D_H
#define _VECT3D_H

/*
	TODO:
		Make entirely quaternion based Vector lib
	For right now though, this works more than addequitly
*/

#include <cmath>
class Matrix;
class Vect3D;
typedef Vect3D Point3D;

struct P3D {   // quick hack to be compatibly with Chris Hecker
	fixed28_4 x,y;
	float z,u,v;
};

class Vect3D {
	friend class Matrix;
protected:
			double x,y,z;
			float u,v;  // currently unused
public:
	Vect3D(): x(0),y(0),z(0),u(0),v(0) {}
	Vect3D(const Vect3D &vec): x(vec.x), y(vec.y), z(vec.z), u(vec.u), v(vec.u) {}
	Vect3D(double X,double Y,double Z, float U=0, float V=0):x(X),y(Y),z(Z),u(U),v(V) {}

	const double&  getx() const {return x;}
	const double&  gety() const {return y;}
	const double&  getz() const {return z;}
	const float&   getu() const {return u;}
	const float&   getv() const {return v;}

	void  set(double X,double Y,double Z, float U=0, float V=0 ) {x=X; y=Y; z=Z; u=U; v=V;}
	void  setx(double val) {x=val;}
	void  sety(double val) {y=val;}
	void  setz(double val) {z=val;}
	void  setu(float val)  {u=val;}
	void  setv(float val)  {v=val;}
	void  insure_0();

	Vect3D operator-() {return Vect3D(-x,-y,-z);}
	Vect3D operator+(const Vect3D &v) const {return Vect3D(x+v.x, y+v.y, z+v.z);}
	Vect3D operator-(const Vect3D &v) const {return Vect3D(x-v.x, y-v.y, z-v.z);}

	Vect3D& operator+=(const Vect3D &v)	{x+=v.x;  y+=v.y; z+=v.z; return *this;}
	Vect3D& operator-=(const Vect3D &v)	{x-=v.x;  y-=v.y; z-=v.z; return *this;}
	Vect3D& operator*=(double scalar)		{x*=scalar; y*=scalar; z*=scalar; return *this;}

	friend Vect3D operator* (double scalar, const Vect3D &v) {return Vect3D(scalar*v.x, scalar*v.y, scalar*v.z);}
	friend Vect3D operator* (const Vect3D &v, double scalar) {return Vect3D(scalar*v.x, scalar*v.y, scalar*v.z);}

	friend ostream &operator<<(ostream &, const Vect3D &);
	friend istream &operator>>(istream &, Vect3D &);

	bool operator!=(const Vect3D &v) const {if (x!=v.x  ||  y!=v.y  ||  z!=v.z) return true; return false;}
	bool operator==(const Vect3D &v) const {if (x==v.x  &&  y==v.y  &&  z==v.z) return true; return false;}
	Vect3D &operator=(const Vect3D &v) {x=v.x; y=v.y; z=v.z; return *this;}

	double dot(const Vect3D &v)	const	{return (x*v.x + y*v.y + z*v.z);}
	double length()							const	{return (sqrt(x*x + y*y + z*z));}

	Vect3D cross(const Vect3D &v) const {return Vect3D(y*v.z - z*v.y, z*v.x - x*v.z,  x*v.y - y*v.x);}

	Vect3D norm();
	Vect3D surface_norm(const Vect3D &, const Vect3D &, const Vect3D &, const Vect3D &) const;
};

void  Vect3D::insure_0() {
	if(x<CLIP_PLANE_EPSILON && x>-CLIP_PLANE_EPSILON) x=0;
	if(y<CLIP_PLANE_EPSILON && y>-CLIP_PLANE_EPSILON) y=0;
	if(z<CLIP_PLANE_EPSILON && z>-CLIP_PLANE_EPSILON) z=0;
}

ostream &operator<<(ostream &out, const Vect3D &v) {
	out.setf(ios::left, ios::adjustfield);
	out << "  " << setprecision(4) << setw(5) << v.x << "  " << setw(5) << v.y << "  " << setw(5) << v.z << '\n';
	return out;
}
istream &operator>>(istream &in, Vect3D &vec) {
	in >> vec.x >> vec.y >> vec.z /* >> vec.u >> vec.v */;
	return in;
}

Vect3D Vect3D::norm() {
	double lngth = length();
	if(lngth == 0) return (*this);
	lngth=1/lngth;
	x *= lngth;
	y *= lngth;
	z *= lngth;
	return *this;
}

Vect3D Vect3D::surface_norm(const Vect3D &v1, const Vect3D &v2, const Vect3D &v3, const Vect3D &v4) const {
	Vect3D dir1,dir2,srfnorm1,srfnorm2;
	double lngth,lngth2;
	dir1 = v2-v1;
	dir2 = v4-v1;
	srfnorm1 = dir1.cross(dir2);
	lngth = srfnorm1.length();
	dir1 = v4-v3;
	dir2 = v2-v3;
	srfnorm2 = dir1.cross(dir2);
	lngth2 = srfnorm2.length();
	if (lngth == 0)
		return (srfnorm2.norm());
	else {
		srfnorm1.norm();
		if (lngth2 != 0) {
			srfnorm2.norm();
			srfnorm1 += srfnorm2;
			srfnorm1 *= 0.5;
		}
	}
	return srfnorm1;
}
#endif
